/*
 * Segment 2 (KEx),  SWE 2.WebFS Web Feature Service
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.wfs.wfs;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.geotools.data.DataStore;
import org.geotools.data.vrz3.VRZ3DataStoreFactory;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Hauptklasse f&uuml;r die SWE 2.WebFS.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: WebFS.java 8187 2008-04-14 16:06:50Z gieseler $
 * 
 */
public class WebFS implements StandardApplication {
	/**
	 * DebugLogger fr Debug-Ausgaben.
	 */
	private static Debug debug;

	/**
	 * MAIN-Methode.
	 * 
	 * @param args
	 *            Argumente
	 */
	public static void main(String[] args) {
		Thread
				.setDefaultUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler() {

					public void uncaughtException(Thread t, Throwable e) {
						e.printStackTrace();
						Logger.getLogger(getClass().getName()).log(Level.INFO,
								"*** Application wird beendet ***", e); //$NON-NLS-1$
					}

				});

		// final ArgumentList argumentList = new ArgumentList(args);
		StandardApplicationRunner.run(new WebFS(), args);
	}

	/**
	 * Datenverteiler-Verbindung.
	 */
	ClientDavConnection dav;

	/**
	 * HTTP-Port.
	 */
	private int httpPort;

	/**
	 * Log-Datei f&uuml;r die Geoserver-Anwendung.
	 */
	// private String _logFile;
	/**
	 * Dateiname der Definitionsdatei f&uuml;r die zu exportierenden Objekte.
	 */
	private String exportObjekteDatei;

	/**
	 * Geoserver-Thread.
	 */
	Thread geoserverthread = null;

	/**
	 * Namespace.
	 */
	String nameSpace = (String) VRZ3DataStoreFactory.PARAM_NAMESPACE.sample;

	/**
	 * DataStore.
	 */
	private DataStore wfs;

	/**
	 * Standard-Konstruktor.
	 */
	protected WebFS() {
		super();
	}

	/**
	 * Erzeugt den DataStore.
	 * 
	 * @throws WFSException
	 *             bei Ausnahmen
	 */
	private void erzeugeDataStore() throws WFSException {
		ClientDavParameters davprm = dav.getClientDavParameters();

		Map<String, Object> params = new HashMap<String, Object>();
		params.put(VRZ3DataStoreFactory.PARAM_DAV.key, davprm
				.getDavCommunicationAddress()
				+ ":" + davprm.getDavCommunicationSubAddress());
		params.put(VRZ3DataStoreFactory.PARAM_DAV_NUTZER.key, davprm
				.getUserName());
		params.put(VRZ3DataStoreFactory.PARAM_DAV_AUTHENTIFIZIERUNG.key, davprm
				.getUserPassword());
		params.put(VRZ3DataStoreFactory.PARAM_EXPORTCLASS.key,
				exportObjekteDatei);

		try {
			params.put(VRZ3DataStoreFactory.PARAM_NAMESPACE.key, new URI(
					nameSpace));
			wfs = (new VRZ3DataStoreFactory()).createDataStore(params);
		} catch (IOException e) {
			throw new WFSException(e.getMessage());
		} catch (URISyntaxException e1) {
			throw new WFSException(e1.getMessage());
		}

	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see de.bsvrz.sys.funclib.application.StandardApplication#initialize(de.bsvrz.dav.daf.main.ClientDavInterface)
	 */
	public void initialize(ClientDavInterface connection) throws Exception {
		debug = Debug.getLogger();
		dav = (ClientDavConnection) connection;

		try {
			startApp();
		} catch (Exception e) {
			debug.error(e.getMessage());
		}
	}

	/**
	 * Wertet die an die Applikation &uuml;bergebenen Kommandozeilenparameter
	 * aus.<br>
	 * Zus&auml;tzlich zu den Standard-Datenverteiler-Argumenten sind folgende
	 * Argumente m&ouml;glich:
	 * <ul>
	 * <li>-HttpPort=&lt;Wert&gt;, Portnummer unter dem der Web Feature Service
	 * ber HTTP angesprochen werden kann (Standardeinstellung ist 8080), Der
	 * Wert fr den Parameter HttpPort ist auf Werte > 1024 eingeschrnkt. Fr
	 * kleinere Werte sind spezielle Rechte zur Installation des entsprechenden
	 * Listeners auf der jeweiligen Betriebssystem-Plattform erforderlich, die
	 * durch diese SWE nicht abgedeckt werden.,</li>
	 * <li>-exportObjekte=&lt;Datei&gt;, Dateiname einer java-Datei mit der
	 * Konfiguration der ber den Web Feature Service bereitzustellenden
	 * Objekttypen, Standard ist die zur SWE gehrende Datei
	 * ExportObjekteDefault</li>
	 * <li>-logFile=&lt;Datei&gt;, Dateiname der Logdatei fr das Modul
	 * WebFeatureServer, Standardwert: Logfile in der Konfiguration des Modules
	 * WebFeatureServer</li>
	 * </ul>
	 * 
	 * @param argumentList
	 *            die Liste der &uuml;bergebenen Argumente
	 * @see de.bsvrz.sys.funclib.application.StandardApplication#parseArguments(de.bsvrz.sys.funclib.commandLineArgs.ArgumentList)
	 * @throws Exception
	 *             bei Ausnahmen
	 */
	public void parseArguments(final ArgumentList argumentList)
			throws Exception {
		httpPort = argumentList.fetchArgument("-HttpPort=8080").intValue();
		exportObjekteDatei = argumentList.fetchArgument(
				"-exportObjekte=de.bsvrz.kex.wfs.wfs.ExportObjekteDefault")
				.asString();
		// _geoServerRoot = argumentList.fetchArgument(
		// "-geoServerRoot=../geoserver").asString();
		// _logFile = argumentList.fetchArgument("-logFile=").asString();
	}

	/**
	 * Startet die Applikation.
	 * 
	 * @throws Exception
	 *             bei Ausnahmen
	 */
	public void startApp() throws Exception {
		try {
			dav.disconnect(false, "");
			debug.info("Applikation wird gestartet"); //$NON-NLS-1$

			// bestimme Ort der Geoserver-Applikation
			URL geoserverurl = ClassLoader.getSystemResource("geoserver/");
			if (geoserverurl == null) {
				throw new WFSException(
						"Die Geoserver-Anwendung kann nicht lokalisiert werden");
			}

			GeoServerApplikation geoserver = new GeoServerApplikation(
					geoserverurl.getFile());

			geoserver.setHttpPort(httpPort);
			ClientDavParameters davprm = dav.getClientDavParameters();
			geoserver.setDavBenutzer(davprm.getUserName());
			geoserver.setDavAuthentifizierung(davprm.getUserPassword());
			geoserver.setDavHost(davprm.getDavCommunicationAddress() + ":"
					+ davprm.getDavCommunicationSubAddress());
			geoserver.setExportObjekteDatei(exportObjekteDatei);

			geoserver.setzeGeoserverParameter();

			erzeugeDataStore();
			geoserver.erzeugeFeatureTypen(wfs);

			geoserverthread = new Thread(geoserver);

			geoserverthread.start();

			if (geoserverthread.isAlive()) {
				Runtime.getRuntime().addShutdownHook(new Thread() {
					@Override
					public void run() {
						debug.info("Abbruch der Applikation");
						if (geoserverthread != null) {
							debug.info("Geoserver wird beendet");
							geoserverthread.interrupt();
						}
					}
				});
				debug.info("warte auf Ende des GeoServer");
				geoserverthread.join();
				debug.info("GeoServer beendet");
			}
		} catch (Exception e) {
			debug.error(e.getMessage());
		}

		if (geoserverthread != null) {
			debug.info("Geoserver wird beendet");
			geoserverthread.interrupt();
			geoserverthread.join();
		}

		debug.info("Applikation beendet"); //$NON-NLS-1$
	}

}
