/*
 * Segment 2 (KEx),  SWE 2.WebFS Web Feature Service
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package org.geotools.data.vrz3;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.geotools.feature.AttributeType;
import org.geotools.feature.AttributeTypeFactory;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;

/**
 * Factory fr die Bildung von Feature-Attributen aus DaV-Attributen.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: VRZ3AttributeFactory.java 8189 2008-04-14 16:07:08Z gieseler $
 * 
 */

public final class VRZ3AttributeFactory {

	/**
	 * Erzeugt eine {@link Data} Instanz.
	 * 
	 * @param dav
	 *            Datenverteiler-Verbindung
	 * @param atg
	 *            Name der Attributgruppe
	 * @return {@link Data} Instanz.
	 */
	private static Data createData(ClientDavConnection dav, String atg) {
		AttributeGroup attributeGroup = dav.getDataModel().getAttributeGroup(
				atg);
		return dav.createData(attributeGroup);
	}

	/**
	 * Erzeugt ein Feature-Attribut.
	 * 
	 * @param atg
	 *            Attributgruppe
	 * @param data
	 *            Data
	 * @return Feature-Attribut
	 */
	private static VRZ3FeatureAttribute erzeugeAttribut(final String atg,
			final Data data) {
		Class<?> typeClass;
		String name = data.getName();
		String defaultvalue = null;
		// de.bsvrz.dav.daf.main.config.AttributeType type =
		// data.getAttributeType();

		VRZ3FeatureAttribute attribute = new VRZ3FeatureAttribute(atg, data
				.getName());

		// if (type instanceof StringAttributeType) {
		// typeClass = String.class;
		// attribute.setValueSetter( new VRZ3ValueSetter(defaultvalue) {
		// protected void stringToValue()
		// throws Exception {
		// objValue = new String(strValue);
		// }
		// }
		// );
		// } else if (type instanceof IntegerAttributeType) {
		// typeClass = Integer.class;
		// IntegerValueRange range = ((IntegerAttributeType) type).getRange();
		// defaultvalue = ((IntegerAttributeType)
		// type).getDefaultAttributeValue();
		// if(range.getConversionFactor() != 0.0) {
		// typeClass = Double.class;
		// attribute.setValueSetter( new VRZ3ValueSetter(defaultvalue) {
		// protected void stringToValue()
		// throws Exception {
		// String dup = new String(strValue);
		// dup.replaceAll(",", ".");
		// objValue = new Double(dup);
		// }
		// }
		// );
		// }
		// else {
		// typeClass = Integer.class;
		// attribute.setValueSetter( new VRZ3ValueSetter(defaultvalue) {
		// protected void stringToValue()
		// throws Exception {
		// objValue = new Integer(strValue);
		// }
		// }
		// );
		// }
		//				
		// } else if (type instanceof DoubleAttributeType) {
		// typeClass = Double.class;
		// } else if (type instanceof TimeAttributeType) {
		// typeClass = Integer.class;
		// } else if (type instanceof ReferenceAttributeType) {
		// typeClass = String.class;
		// attribute.setValueSetter( new VRZ3ValueSetter(defaultvalue) {
		// protected void stringToValue()
		// throws Exception {
		// objValue = new String(strValue);
		// }
		// }
		// );
		// // result = new ObjektReferenz(data.getName());
		// // SystemObject ref = data.asReferenceValue().getSystemObject();
		// // if (ref == null) {
		// // ((ObjektReferenz) result).set(0L);
		// // } else {
		// // ((ObjektReferenz) result).set(ref.getId());
		// // }
		// }
		// else
		// return null;

		// AttributeType at = AttributeTypeFactory.newAttributeType(name,
		// typeClass, false, size, defaultvalue);

		typeClass = String.class;
		attribute.setValueSetter(new VRZ3ValueSetter(defaultvalue) {
			@Override
			protected void stringToValue() throws Exception {
				objValue = new String(strValue);
			}
		});
		AttributeType at = AttributeTypeFactory.newAttributeType(name,
				typeClass, false, 0, defaultvalue);

		attribute.set_geotoolsAttributeType(at);

		return attribute;
	}

	/**
	 * Erzeugt die Feature-Attribute zu einer Attributgruppe.
	 * 
	 * @param dav
	 *            Datenverteiler-Verbindung
	 * @param atg
	 *            Attributgruppe
	 * @return Collection von Featurte-Attributen
	 */
	public static Collection<? extends VRZ3FeatureAttribute> erzeugeAttribute(
			ClientDavConnection dav, String atg) {
		List<VRZ3FeatureAttribute> atts = new ArrayList<VRZ3FeatureAttribute>();

		Data data = createData(dav, atg);

		if (data.isPlain()) {
			atts.add(erzeugeAttribut(atg, data));
		} else if (data.isArray()) {
			data.getName().contains("a"); // dummy-Anweisung

			/* Iterator zum Durchlaufen des Feldes */
			// Iterator i = data.iterator();
			//
			// /* Name dieses Datums */
			// // String name = data.getName();
			//
			// while (i.hasNext()) {
			// atts.add(getAttribute((Data) i.next()));
			// }
		} else if (data.isList()) {
			/* Iterator zum Durchlaufen der Liste */
			Iterator<?> i = data.iterator();

			/* Name dieses Datums */
			// String name = data.getName();
			// List<AttributeType> listlist = new ArrayList<AttributeType>();
			while (i.hasNext()) {
				// listlist.add(getAttribute((Data) i.next()));
				atts.add(erzeugeAttribut(atg, (Data) i.next()));
			}

			// lat = new ListAttributeType(name, false, listlist.toArray(new
			// AttributeType[listlist.size()]));
			// attlist.add(lat);

			// attlist.addAll(listlist);
		}

		return atts;
	}

	/**
	 * Erzeugt allgemeine Feature-Attribute, die von allen Objekten benutzt
	 * werden.
	 * 
	 * @return Liste von Feature-Attributen
	 */
	static List<VRZ3FeatureAttribute> erzeugeObjektAttribute() {
		List<VRZ3FeatureAttribute> atts = new ArrayList<VRZ3FeatureAttribute>();
		Class<?> typeClass;

		// Name
		VRZ3FeatureAttribute attribute = new VRZ3FeatureAttribute("Name", null);
		typeClass = String.class;
		attribute.setValueSetter(new VRZ3ValueSetter("") {
			@Override
			protected void stringToValue() throws Exception {
				objValue = new String(strValue);
			}
		});
		AttributeType at = AttributeTypeFactory.newAttributeType("Name",
				typeClass, false, 0, "");
		attribute.set_geotoolsAttributeType(at);
		atts.add(attribute);

		// PID
		attribute = new VRZ3FeatureAttribute("PID", null);
		typeClass = String.class;
		attribute.setValueSetter(new VRZ3ValueSetter("") {
			@Override
			protected void stringToValue() throws Exception {
				objValue = new String(strValue);
			}
		});
		at = AttributeTypeFactory.newAttributeType("PID", typeClass, false, 0,
				"");
		attribute.set_geotoolsAttributeType(at);
		atts.add(attribute);

		return atts;
	}

	/**
	 * Privater Konstruktor.
	 */
	private VRZ3AttributeFactory() {
		/** nur zur Beruhigung von CheckStyle */
	}
}
